<?php
namespace Teknisa\CMP\Controllers;

class FechamentoAF {

    protected $fechamentoAFService;
    protected $inconsistenciaAF;
    protected $contaPagar;
    
    public function __construct($fechamentoAFService, $inconsistenciaAF, $contaPagar) {
        $this->fechamentoAFService = $fechamentoAFService;
        $this->inconsistenciaAF = $inconsistenciaAF;
        $this->contaPagar = $contaPagar;
    }


    public function stringParaData($data) {
        if(gettype($data) === 'string') {
            $data = substr($data, 0, 10);
            $data = str_replace('/', '-', $data);
            $data = new \DateTime($data);
        }
        
        return $data;
    }
    
    
    /**
     * Metodo principal para executar o fechamento da AF.
     * 
     * @param {string} $cdfildest Filial de destino da AF
     * @param {string} $cdfillanc Filial de lançamento da AF
     * @param {string} $nrautoforn Númeor da AF
     * @param {string} $cdfilcent Central de compras da filial de destino
     * 
     * @return Caso de errado, retorna uma mensagem de erro. Caso não exista erros,
     * retorna false.
     * 
     */
    public function fechaAF($cdfildest, $cdfillanc, $nrautoforn, $cdfilcent) {
        try {
            $existeAF = $this->fechamentoAFService->verificaExisteAF($cdfildest, $cdfillanc, $nrautoforn);
    
            if($existeAF) {
                
                $this->fechamentoAFService->proceFechamentoAF($cdfildest, $cdfillanc, $nrautoforn);
                
                $this->inconsistenciaAF->verificaInconsistencias($cdfildest, $cdfillanc, $nrautoforn, $cdfilcent, true);
                
                self::verificaInconsFornExtra($cdfildest, $cdfillanc, $nrautoforn);
                
                self::verificaInconsPrecoAnterior($cdfildest, $cdfillanc, $nrautoforn);
                
                self::geraContaPagarOrcFinan($cdfildest, $cdfillanc, $nrautoforn);
                
            }
            
            return false;
            
        } catch(\Exception $e) {
            $code = $e->getCode();
            
            switch ($code) {
                case 10:
                    throw new \Exception("AF não fechada. Ocorreu um erro ao tentar excluir conta a pagar relacionada.");
                    break;
                    
                case 20:
                    return "AF fechada. Não foi possível gerar conta a pagar.";
                    break;
                    
                case 30:
                    throw $e;
                    break;
                    
                default:
                    throw $e;
                    break;
            }
        }
        
    }
    
    
    public function verificaInconsFornExtra($cdfildest, $cdfillanc, $nrautoforn) {
        
        $fornExtra = $this->fechamentoAFService->retornaForneExtra($cdfildest, $cdfillanc, $nrautoforn);

        if($fornExtra && $fornExtra['IDAUTOFORNEX'] == 'S' && !$fornExtra['AUXFORNEXTRA']) {
            
            $dsinconsaf = 'Fornecedor não é extra.';
            $idsituatuaaf = '1';
            
            $this->fechamentoAFService->atualizaInconsAF($cdfildest, $cdfillanc, $nrautoforn, $idsituatuaaf, $dsinconsaf);
            
        }
        
    }
    
    
    public function verificaInconsPrecoAnterior($cdfildest, $cdfillanc, $nrautoforn) {
        
        $idinconsprecsup = $this->fechamentoAFService->verificaInconsPreco($cdfildest)['IDINCONSPRECSUP'];

        if($idinconsprecsup == 'S') {
            
            $itensAF = $this->fechamentoAFService->retornaPrecoItensAF($cdfildest, $cdfillanc, $nrautoforn);

            foreach($itensAF as $itemAF) {

                if($itemAF['VRUNITITEM']) {
                    
                    if($itemAF['VRUNITITEM'] < ($itemAF['VRPREUNIAF'] / $itemAF['VRFATOCONV'])) {

                        $dsinconsitaf = 'Preço unitário (unidade de estoque) ' . $itemAF['VRPREUNIAF'] . ' maior que o último preço utilizado (unidade de estoque) ' . $itemAF['VRUNITITEM'];
                        $this->fechamentoAFService->atualizaInconsItemAF($cdfildest, $cdfillanc, $nrautoforn, $itemAF['CDPRODUTO'], $itemAF['CDMARCA'], self::stringParaData($itemAF['DTPROGENTAF']), $dsinconsitaf);
                        
                    }
                    
                    $idsituatuaaf = '1';
                    $this->fechamentoAFService->atualizaInconsAF($cdfildest, $cdfillanc, $nrautoforn, $idsituatuaaf);
                }
                
            }
            
        }
        
    }
    
    
    public function geraContaPagarOrcFinan($cdfildest, $cdfillanc, $nrautoforn) {
        
        try {
            
            $this->fechamentoAFService->atualizaTipoContaPagar($cdfildest, $cdfillanc, $nrautoforn);
            
            $geraNF = $this->contaPagar->excluiNFContaPagar($cdfildest, $cdfillanc, $nrautoforn, true);
            if(!$geraNF) {
                
                throw new \Exception("AF não fechada. Orçamento financeiro foi ultrapassdo.", 30);
                
            } else {
                
                $this->contaPagar->geraNFContaPagar($cdfildest, $cdfillanc, $nrautoforn);
                
            }
            
        } catch(\Exception $e) {
            
           throw $e;
            
        }
        
    }
    
}
   